Package: RSQLite
Version: 1.1-2
Date: 2017-01-07
Title: 'SQLite' Interface for R
Authors@R: c(
    person("Kirill", "Müller", role = c("aut", "cre"), email = "krlmlr+r@mailbox.org"),
    person("Hadley", "Wickham", role = c("aut")),
    person(c("David", "A."), "James", role = "aut"),
    person("Seth", "Falcon", role = "aut"),
    person(family = "SQLite Authors", role = "ctb", comment = "for the included SQLite sources"),
    person("Liam", "Healy", role = "ctb", comment = "for the included SQLite sources"),
    person(family = "R Consortium", role = "cph"),
    person(family = "RStudio", role = "cph")
    )
Description: Embeds the 'SQLite' database engine in R and
    provides an interface compliant with the 'DBI' package. The
    source for the 'SQLite' engine (version 3.8.8.2) is included.
Depends: R (>= 3.1.0)
Suggests: DBItest, knitr, rmarkdown, testthat
Imports: DBI (>= 0.4-9), memoise, methods, Rcpp (>= 0.12.7)
LinkingTo: Rcpp, BH, plogr
Encoding: UTF-8
License: LGPL (>= 2)
URL: https://github.com/rstats-db/RSQLite
BugReports: https://github.com/rstats-db/RSQLite/issues
Collate: 'RcppExports.R' 'SQLiteConnection.R' 'SQLiteDriver.R'
        'SQLiteResult.R' 'connect.R' 'copy.R' 'datasetsDb.R'
        'deprecated.R' 'dummy.R' 'extensions.R' 'query.R' 'rownames.R'
        'table.R' 'transactions.R' 'utils.R' 'zzz.R'
VignetteBuilder: knitr
RoxygenNote: 5.0.1.9000
NeedsCompilation: yes
Packaged: 2017-01-08 00:12:25 UTC; muelleki
Author: Kirill Müller [aut, cre],
  Hadley Wickham [aut],
  David A. James [aut],
  Seth Falcon [aut],
  SQLite Authors [ctb] (for the included SQLite sources),
  Liam Healy [ctb] (for the included SQLite sources),
  R Consortium [cph],
  RStudio [cph]
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2017-01-08 16:57:09
Built: R 3.4.1; x86_64-w64-mingw32; 2017-08-21 02:13:11 UTC; windows
Archs: x64
