Package: RSQLite
Title: SQLite Interface for R
Version: 3.53.1
Date: 2026-05-22
Authors@R: c(
    person("Kirill", "Müller", , "kirill@cynkra.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1416-3412")),
    person("Hadley", "Wickham", role = "aut"),
    person(c("David", "A."), "James", role = "aut"),
    person("Seth", "Falcon", role = "aut"),
    person("D. Richard", "Hipp", role = "ctb",
           comment = "for the included SQLite sources"),
    person("Dan", "Kennedy", role = "ctb",
           comment = "for the included SQLite sources"),
    person("Joe", "Mistachkin", role = "ctb",
           comment = "for the included SQLite sources"),
    person(, "SQLite Authors", role = "ctb",
           comment = "for the included SQLite sources"),
    person("Liam", "Healy", role = "ctb",
           comment = "for the included SQLite sources"),
    person("R Consortium", role = "fnd"),
    person(, "RStudio", role = "cph")
  )
Description: Embeds the SQLite database engine in R and provides an
    interface compliant with the DBI package. The source for the SQLite
    engine and for various extensions is included.
    System libraries will never be consulted because this package relies
    on static linking for the plugins it includes; this also ensures a
    consistent experience across all installations.
License: LGPL (>= 2.1)
URL: https://rsqlite.r-dbi.org, https://github.com/r-dbi/RSQLite
BugReports: https://github.com/r-dbi/RSQLite/issues
Depends: R (>= 3.1.0)
Imports: bit64, blob (>= 1.2.0), DBI (>= 1.2.0), memoise, methods,
        pkgconfig, rlang
Suggests: callr, cli, DBItest (>= 1.8.0), decor, gert, gh, hms, knitr,
        magrittr, rmarkdown, rvest, testthat (>= 3.0.0), withr, xml2
LinkingTo: cpp11 (>= 0.4.0)
VignetteBuilder: knitr
Config/Needs/website: r-dbi/dbitemplate
Config/autostyle/scope: line_breaks
Config/autostyle/strict: false
Config/testthat/edition: 3
Encoding: UTF-8
Collate: 'SQLiteConnection.R' 'SQLKeywords_SQLiteConnection.R'
        'SQLiteDriver.R' 'SQLite.R' 'SQLiteResult.R' 'coerce.R'
        'compatRowNames.R' 'copy.R' 'cpp11.R' 'datasetsDb.R'
        'dbAppendTable_SQLiteConnection.R' 'dbBeginTransaction.R'
        'dbBegin_SQLiteConnection.R' 'dbBind_SQLiteResult.R'
        'dbClearResult_SQLiteResult.R' 'dbColumnInfo_SQLiteResult.R'
        'dbCommit_SQLiteConnection.R' 'dbConnect_SQLiteConnection.R'
        'dbConnect_SQLiteDriver.R' 'dbDataType_SQLiteConnection.R'
        'dbDataType_SQLiteDriver.R' 'dbDisconnect_SQLiteConnection.R'
        'dbExistsTable_SQLiteConnection_Id.R'
        'dbExistsTable_SQLiteConnection_character.R'
        'dbFetch_SQLiteResult.R' 'dbGetException_SQLiteConnection.R'
        'dbGetInfo_SQLiteConnection.R' 'dbGetInfo_SQLiteDriver.R'
        'dbGetPreparedQuery.R'
        'dbGetPreparedQuery_SQLiteConnection_character_data.frame.R'
        'dbGetRowCount_SQLiteResult.R'
        'dbGetRowsAffected_SQLiteResult.R'
        'dbGetStatement_SQLiteResult.R' 'dbHasCompleted_SQLiteResult.R'
        'dbIsValid_SQLiteConnection.R' 'dbIsValid_SQLiteDriver.R'
        'dbIsValid_SQLiteResult.R' 'dbListObjects_SQLiteConnection.R'
        'dbListResults_SQLiteConnection.R'
        'dbListTables_SQLiteConnection.R'
        'dbQuoteIdentifier_SQLiteConnection_SQL.R'
        'dbQuoteIdentifier_SQLiteConnection_character.R'
        'dbReadTable_SQLiteConnection_character.R'
        'dbRemoveTable_SQLiteConnection_character.R'
        'dbRollback_SQLiteConnection.R' 'dbSendPreparedQuery.R'
        'dbSendPreparedQuery_SQLiteConnection_character_data.frame.R'
        'dbSendQuery_SQLiteConnection_character.R'
        'dbUnloadDriver_SQLiteDriver.R'
        'dbUnquoteIdentifier_SQLiteConnection_SQL.R'
        'dbWriteTable_SQLiteConnection_character_character.R'
        'dbWriteTable_SQLiteConnection_character_data.frame.R'
        'db_bind.R' 'deprecated.R' 'export.R' 'fetch_SQLiteResult.R'
        'import-standalone-check_suggested.R'
        'import-standalone-purrr.R' 'initExtension.R' 'initRegExp.R'
        'isSQLKeyword_SQLiteConnection_character.R'
        'make.db.names_SQLiteConnection_character.R' 'pkgconfig.R'
        'show_SQLiteConnection.R' 'sqlData_SQLiteConnection.R'
        'table.R' 'transactions.R' 'utils.R' 'version.R' 'zzz.R'
Config/roxygen2/version: 8.0.0.9000
NeedsCompilation: yes
Packaged: 2026-05-22 15:59:59 UTC; kirill
Author: Kirill Müller [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1416-3412>),
  Hadley Wickham [aut],
  David A. James [aut],
  Seth Falcon [aut],
  D. Richard Hipp [ctb] (for the included SQLite sources),
  Dan Kennedy [ctb] (for the included SQLite sources),
  Joe Mistachkin [ctb] (for the included SQLite sources),
  SQLite Authors [ctb] (for the included SQLite sources),
  Liam Healy [ctb] (for the included SQLite sources),
  R Consortium [fnd],
  RStudio [cph]
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2026-05-23 05:10:57 UTC
Built: R 4.6.0; i486--netbsdelf; 2026-06-20 21:29:00 UTC; unix
