# Generated by roxygen2: do not edit by hand

S3method(format,SQLiteConnection)
export(Id)
export(SQLITE_RO)
export(SQLITE_RW)
export(SQLITE_RWC)
export(SQLite)
export(datasetsDb)
export(dbBeginTransaction)
export(dbBuildTableDefinition)
export(dbGetPreparedQuery)
export(dbSendPreparedQuery)
export(initExtension)
export(initRegExp)
export(isIdCurrent)
export(rsqliteVersion)
export(sqliteBuildTableDefinition)
export(sqliteCopyDatabase)
export(sqliteQuickColumn)
exportClasses(SQLiteConnection)
exportClasses(SQLiteDriver)
exportClasses(SQLiteResult)
exportMethods(SQLKeywords)
exportMethods(dbAppendTable)
exportMethods(dbBegin)
exportMethods(dbBind)
exportMethods(dbCanConnect)
exportMethods(dbClearResult)
exportMethods(dbColumnInfo)
exportMethods(dbCommit)
exportMethods(dbConnect)
exportMethods(dbCreateTable)
exportMethods(dbDataType)
exportMethods(dbDisconnect)
exportMethods(dbDriver)
exportMethods(dbExecute)
exportMethods(dbExistsTable)
exportMethods(dbFetch)
exportMethods(dbGetException)
exportMethods(dbGetInfo)
exportMethods(dbGetPreparedQuery)
exportMethods(dbGetQuery)
exportMethods(dbGetRowCount)
exportMethods(dbGetRowsAffected)
exportMethods(dbGetStatement)
exportMethods(dbHasCompleted)
exportMethods(dbIsReadOnly)
exportMethods(dbIsValid)
exportMethods(dbListFields)
exportMethods(dbListObjects)
exportMethods(dbListResults)
exportMethods(dbListTables)
exportMethods(dbQuoteIdentifier)
exportMethods(dbQuoteLiteral)
exportMethods(dbQuoteString)
exportMethods(dbReadTable)
exportMethods(dbRemoveTable)
exportMethods(dbRollback)
exportMethods(dbSendPreparedQuery)
exportMethods(dbSendQuery)
exportMethods(dbSendStatement)
exportMethods(dbUnloadDriver)
exportMethods(dbUnquoteIdentifier)
exportMethods(dbWithTransaction)
exportMethods(dbWriteTable)
exportMethods(fetch)
exportMethods(isSQLKeyword)
exportMethods(make.db.names)
exportMethods(show)
exportMethods(sqlData)
import(DBI)
import(methods)
importFrom(Rcpp,sourceCpp)
importFrom(bit64,integer64)
importFrom(bit64,is.integer64)
importFrom(blob,blob)
importFrom(memoise,memoise)
useDynLib(RSQLite, .registration = TRUE)
